-- buildsys macros routines, safe to use at SRPM build stage

local fedora = require "fedora.common"

local namespace = "buildsys"
local rads = {
   ["start"] = {"name"},
   ["evr"]   = {"epoch", "version", "release", "post_release"},
   ["end"]   = {"summary", "license", "url", "tags", "description"},
}

local function radicals()
  local R = {}
  for k, v in pairs(rads) do
    R[k] = v
  end
  local all = {}
  for k, v in pairs(R) do
    all = fedora.mergelists({all, v})
  end
  R["all"] = all
  return R
end

local function srpm_pkg_start(verbose)
  print(fedora.expand([[
Name:           %{source_name}
]]))
  -- Name: is declared, %{_sourcedir} can be used
  local sn = "source"
  local myrads = radicals()
  fedora.mset(myrads["evr"], namespace, '', {
                fedora.mread(myrads["evr"], sn, '', false),
              }, verbose, false)
  local changelog = ''
  local changelog_file = '%{_sourcedir}/%{' .. namespace .. '_changelog_file}'
  local f = io.open(fedora.expand(changelog_file), "r")
  if f then
    for l in f:lines() do
      changelog = changelog .. l .. "\n"
    end
    io.close(f)
  end
  fedora.set(namespace .. "_changelog_content", changelog, verbose)
  print(fedora.expand([[
%{?buildsys_epoch:Epoch:          %{buildsys_epoch}}
Version:        %{buildsys_version}
Release:        %{buildsys_release}%{?dist}%{?buildsys_post_release:.%{buildsys_post_release}}]]))
  -- Make sure the environment is ready for eventual pkg_end() execution
  -- pkg_end is not srpm specific and does not operate from the source namespace
  local cn = "__current_" .. namespace
  fedora.mset(myrads["end"], cn, '', {
                fedora.mread(myrads["end"], sn, '', false),
              }, verbose, true)
  fedora.set("__" .. namespace .. "_srpm_done", true, verbose)
end

local function pkg_end()
  local cn = "__current_" .. namespace
  local itags = {}
  for t, m in pairs({["Epoch"] = "epoch", ["Version"] = "version"}) do
    local v = fedora.read(cn .. '_' .. m)
    if v and (v ~= fedora.read('source_' .. m)) then
      table.insert(itags, t .. ': ' .. v)
    end
  end
  print(rpm.expand([[

Summary:        %{__current_buildsys_summary}
]] .. table.concat(itags, '\n') .. [[
%{?__current_buildsys_license:License:        %{__current_buildsys_license}}
%{?__current_buildsys_url:URL:            %{__current_buildsys_url}}
%{?__current_buildsys_tags}
%description -n %{__current_buildsys_name}
%wordwrap    -v   __current_buildsys_description
]]))
end

-- %new_package core
local function new_package(pkg_name, name_suffix, verbose)
  local sn = "source"
  local cn = "__current_" .. namespace
  local source_name = fedora.read(sn .. "_name")
  -- Safety net when the wrapper is used in conjunction with traditional syntax
  if fedora.read("name") and (not source_name) then
    warning([[
Something already set a package name. However, %%{source_name} is not set.
Please set %%{source_name} to the SRPM name to ensure reliable processing.
]])
    if name_suffix then
      print(fedora.expand("%package        " .. name_suffix))
    else
      print(fedora.expand("%package     -n " .. pkg_name))
    end
    return
  end
  -- New processing
  -- We can not rely on cn .. "_name" because new_package (unlike pkg)
  -- does not require users to clear past environement between calls
  if not (pkg_name or name_suffix or source_name) then
    fedora.error(
      "You need to set %%{source_name} or provide explicit package naming!")
  end
  local first = not fedora.read("__" .. namespace .. "_srpm_done")
  if name_suffix then
    pkg_name = "%{source_name}-" .. name_suffix
    print(fedora.expand("%package        "  .. name_suffix))
  else
    if first and not source_name then
      source_name = pkg_name
      fedora.safeset(sn .. "_name", source_name, verbose)
    end
    if not pkg_name then
      pkg_name = source_name
    end
    if (pkg_name == source_name) then
      local myrads = radicals()
      -- Collect bits that may have been declared in the __current_
      -- namespace. srpm_pkg_start will take the source namespace as only
      -- trusted reference
      fedora.mset(myrads["all"], sn, "", {
          -- cn is a temporary reusable ref, all mreads must resolve
          fedora.mread({"epoch", "version"}, cn, '', true),
          fedora.mread(myrads["end"], cn, '', true),
        }, verbose, false)
      srpm_pkg_start()
    else
      print(fedora.expand("%package     -n " .. pkg_name))
    end
  end
  _ = fedora.suffix({[namespace .. "_name"] = pkg_name}, true, verbose)
  local cn = "__current_" .. namespace
  fedora.set(cn .. "_name", pkg_name, verbose)
end

local function pkg(verbose)
  local cn = "__current_" .. namespace
  local pkg_name = fedora.read(cn .. "_name")
  new_package(pkg_name, nil, verbose)
  pkg_end()
end

local function reset(verbose)
  local cn = "__current_" .. namespace
  local myrads = radicals()
  fedora.unset("__buildsys_srpm_done", verbose)
  fedora.mset(myrads["all"], cn, '', {{}}, verbose, true)
end

return {
  namespace   = namespace,
  radicals    = radicals()['all'],
  new_package = new_package,
  pkg         = pkg,
  reset       = reset,
}
